package CoswayUtil;

import net.milkbowl.vault.economy.Economy;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.plugin.java.JavaPlugin;

import java.text.DecimalFormat;

public class BankingSystem implements Listener {
    private final CoswayUtil plugin;
    private final Economy economy;
    private final NamespacedKey recipientKey;
    private final NamespacedKey amountKey;

    public BankingSystem(CoswayUtil plugin, Economy economy) {
        this.plugin = plugin;
        this.economy = economy;
        this.recipientKey = new NamespacedKey(plugin, "check_recipient");
        this.amountKey = new NamespacedKey(plugin, "check_amount");
        Bukkit.getPluginManager().registerEvents(this, plugin);
    }
    private final DecimalFormat formatter = new DecimalFormat("#,###.##");
    public void writeCheck(Player sender, String recipientName, double amount) {
        Player recipient = Bukkit.getPlayer(recipientName);
        if (recipient == null) {
            sender.sendMessage(ChatColor.RED + "Player not found!");
            return;
        }
        if (amount <= 0) {
            sender.sendMessage(ChatColor.RED + "Invalid check amount!");
            return;
        }
        if (!economy.has(sender, amount)) {
            sender.sendMessage(ChatColor.RED + "You do not have enough funds to cover this check.");
            return;
        }

        economy.withdrawPlayer(sender, amount);
        ItemStack check = new ItemStack(Material.PAPER);
        ItemMeta meta = check.getItemMeta();
        if (meta != null) {
            // Format the amount for better readability
            String formattedAmount = formatter.format(amount);
            meta.setDisplayName(ChatColor.GOLD + "C.S.E.L. Check");
            meta.getPersistentDataContainer().set(recipientKey, PersistentDataType.STRING, recipientName);
            meta.getPersistentDataContainer().set(amountKey, PersistentDataType.DOUBLE, amount);
            check.setItemMeta(meta);
        }
        // Format the amount for better readability
        String formattedAmount = formatter.format(amount);
        sender.getInventory().addItem(check);
        sender.sendMessage(ChatColor.GREEN + "You have written a check for " + ChatColor.GOLD + "$" + formattedAmount + ChatColor.GREEN + " to " + recipientName);
    }

    @EventHandler
    public void onCheckUse(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        ItemStack item = player.getInventory().getItemInMainHand();
        if (item.getType() != Material.PAPER || !item.hasItemMeta()) return;

        ItemMeta meta = item.getItemMeta();
        if (meta == null) return;

        PersistentDataContainer pdc = meta.getPersistentDataContainer();
        if (!pdc.has(recipientKey, PersistentDataType.STRING) || !pdc.has(amountKey, PersistentDataType.DOUBLE)) return;

        String recipientName = pdc.get(recipientKey, PersistentDataType.STRING);
        double amount = pdc.get(amountKey, PersistentDataType.DOUBLE);

        if (!player.getName().equalsIgnoreCase(recipientName)) {
            player.sendMessage(ChatColor.RED + "This check is not written to you!");
            return;
        }

        economy.depositPlayer(player, amount);
        player.getInventory().getItemInMainHand().setAmount(0);
        player.sendMessage(ChatColor.GREEN + "You have cashed a check for " + ChatColor.GOLD + "$" + amount);
    }
}
