package CoswayUtil;

import net.milkbowl.vault.economy.Economy;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;

import java.text.DecimalFormat;
import java.util.Arrays;

public class CheckCommand implements CommandExecutor {
    private final CoswayUtil plugin;
    private final Economy econ;
    private final NamespacedKey recipientKey;
    private final NamespacedKey amountKey;

    public CheckCommand(CoswayUtil plugin, Economy econ) {
        this.plugin = plugin;
        this.econ = econ;
        this.recipientKey = new NamespacedKey(plugin, "check_recipient");
        this.amountKey = new NamespacedKey(plugin, "check_amount");
    }

    @Override
    public boolean onCommand(CommandSender sender, Command cmd, String label, String[] args) {
        if (!(sender instanceof Player player)) {
            sender.sendMessage(BankName()+ChatColor.RED+"Only players can use this command.");
            return true;
        }

        if (cmd.getName().equalsIgnoreCase("writecheck")) {
            if (args.length != 2) {
                player.sendMessage(BankName()+ChatColor.GOLD+"Usage: /writecheck <player> <amount>");
                return true;
            }

            Player recipient = Bukkit.getPlayerExact(args[0]);
            if (recipient == null) {
                player.sendMessage(BankName()+ChatColor.RED+"Player not found!");
                return true;
            }

            double amount;
            try {
                amount = Double.parseDouble(args[1]);
                if (amount <= 0) {
                    player.sendMessage(BankName()+ChatColor.GOLD+"Amount must be positive.");
                    return true;
                }
            } catch (NumberFormatException e) {
                player.sendMessage(BankName()+ChatColor.RED+"Invalid amount.");
                return true;
            }

            if (!econ.has(player, amount)) {
                player.sendMessage(BankName()+ChatColor.RED+"Insufficient funds.");
                return true;
            }
            // Format the amount for better readability
            String formattedAmount = formatter.format(amount);
            econ.withdrawPlayer(player, amount);
            player.getInventory().addItem(createCheck(recipient.getName(), amount));
            sender.sendMessage(BankName()+ChatColor.GREEN + "You have written a check for " + ChatColor.GOLD + "$" + formattedAmount + ChatColor.GREEN + " to " + recipient.getName());
            return true;
        }

        if (cmd.getName().equalsIgnoreCase("cashcheck")) {
            ItemStack check = player.getInventory().getItemInMainHand();
            if (!isCheck(check)) {
                player.sendMessage(BankName()+ChatColor.RED+"You must hold a valid check to cash it.");
                return true;
            }

            ItemMeta meta = check.getItemMeta();
            String recipient = meta.getPersistentDataContainer().get(recipientKey, PersistentDataType.STRING);
            Double amount = meta.getPersistentDataContainer().get(amountKey, PersistentDataType.DOUBLE);

            if (recipient == null || amount == null) {
                player.sendMessage(BankName()+ChatColor.RED+"Invalid check.");
                return true;
            }

            if (!player.getName().equalsIgnoreCase(recipient)) {
                player.sendMessage(BankName()+ChatColor.RED + "This check is not written to you!");
                return true;
            }

            econ.depositPlayer(player, amount);
            player.getInventory().setItemInMainHand(null);
            player.sendMessage(BankName()+ChatColor.GREEN + "You have cashed a check for " + ChatColor.GOLD + "$" + amount);
            return true;
        }

        return false;
    }
    private final DecimalFormat formatter = new DecimalFormat("#,###.##");
    public String BankName() {
        return ChatColor.GRAY+"[  "+ChatColor.AQUA+"C.S.E.L. Bank "+ChatColor.GRAY+"] ";
    }
    private ItemStack createCheck(String recipient, double amount) {
        ItemStack check = new ItemStack(Material.PAPER);
        ItemMeta meta = check.getItemMeta();
        // Format the amount for better readability
        String formattedAmount = formatter.format(amount);
        meta.setDisplayName("§6C.S.E.L. Check");
        meta.setLore(Arrays.asList("§7Recipient: §b" + recipient, "§7Amount: §a$" + formattedAmount));

        meta.getPersistentDataContainer().set(recipientKey, PersistentDataType.STRING, recipient);
        meta.getPersistentDataContainer().set(amountKey, PersistentDataType.DOUBLE, amount);

        check.setItemMeta(meta);
        return check;
    }

    private boolean isCheck(ItemStack item) {
        if (item == null || item.getType() != Material.PAPER || !item.hasItemMeta()) {
            return false;
        }
        return item.getItemMeta().getPersistentDataContainer().has(recipientKey, PersistentDataType.STRING);
    }
}
