package CoswayUtil;

import org.bukkit.*;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.*;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.ProjectileLaunchEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;

import java.util.Comparator;
import java.util.List;

public class TracerArrowListener implements Listener {

    private final CoswayUtil plugin;

    public TracerArrowListener(CoswayUtil plugin) {
        this.plugin = plugin;
        Bukkit.getPluginManager().registerEvents(this, plugin);
    }

    @EventHandler
    public void onArrowShoot(ProjectileLaunchEvent event) {
        if (!(event.getEntity() instanceof Arrow arrow)) return;
        if (!(arrow.getShooter() instanceof Player player)) return;
        if (!player.getInventory().getItemInMainHand().containsEnchantment(Enchantment.LOYALTY)) return;

        new BukkitRunnable() {
            @Override
            public void run() {
                if (arrow.isDead() || arrow.isInBlock()) {
                    cancel();
                    return;
                }

                LivingEntity target = getNearestEntity(arrow);
                if (target == null) return;

                // Adjust trajectory to aim at target
                Vector newVelocity = target.getLocation().toVector().subtract(arrow.getLocation().toVector()).normalize().multiply(3.0);
                arrow.setVelocity(newVelocity);

                // Visual and sound feedback
                arrow.getWorld().spawnParticle(Particle.DUST, arrow.getLocation(), 5, 0.1, 0.1, 0.1, 0, new Particle.DustOptions(Color.ORANGE, 1));
                arrow.getWorld().playSound(arrow.getLocation(), Sound.ENTITY_ARROW_HIT_PLAYER, 0.5f, 1.5f);
            }
        }.runTaskTimer(plugin, 5L, 2L); // Starts after 5 ticks, updates every 2 ticks
    }
    public static ItemStack getTracer() {
        ItemStack wand = new ItemStack(Material.BOW);
        ItemMeta meta = wand.getItemMeta();
        if (meta != null) {
            meta.setDisplayName(ChatColor.YELLOW + "Tracer Bow");
            meta.setLore(java.util.Arrays.asList(ChatColor.GOLD + "Automatically home on hostiles", ChatColor.GRAY + "Homing Range: 25 blocks"));
            meta.setUnbreakable(true);
            meta.addEnchant(Enchantment.LOYALTY,1,true);
            meta.setEnchantable(10);
            wand.setItemMeta(meta);
        }
        return wand;
    }
    private LivingEntity getNearestEntity(Arrow arrow) {
        List<Entity> entities = arrow.getNearbyEntities(25, 25, 25);

            return entities.stream()
                    .filter(e -> e instanceof LivingEntity && !(e instanceof Player) && (e instanceof Monster))
                    .map(e -> (LivingEntity) e)
                    .min(Comparator.comparingDouble(e -> e.getLocation().distanceSquared(arrow.getLocation())))
                    .orElse(null);
        }
}


